﻿using System;
using System.IO.Ports;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Diagnostics;
using System.IO;

namespace org.mbed.RPC
{
    // *  This class is used to create an object which can communicate using RPC to an mbed Connected over Serial using RXTX.
    // *  This class requires RXTX to be installed on your computer:  http://www.rxtx.org/
    // * 
    // * @author Michael Walker
    // * @license
    // * Copyright (c) 2010 ARM Ltd
    // *
    // *Permission is hereby granted, free of charge, to any person obtaining a copy
    // *of this software and associated documentation files (the "Software"), to deal
    // *in the Software without restriction, including without limitation the rights
    // *to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
    // *copies of the Software, and to permit persons to whom the Software is
    // *furnished to do so, subject to the following conditions:
    // * <br>
    // *The above copyright notice and this permission notice shall be included in
    // *all copies or substantial portions of the Software.
    // * <br>
    // *THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
    // *IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
    // *FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
    // *AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
    // *LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
    // *OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
    // *THE SOFTWARE.
    public class SerialRxTxRPC : mbed
    {
        protected static SerialPort mbedPort;

        // * Create an mbed object to control an mbed connected over serial.
        // * <br>
        // * This class requires RxTx to be installed.
        // *  
        // * @param PortName The serial port mbed is connected to eg "COM5"
        // * @param Baud The baud rate for the serial communication
	    public SerialRxTxRPC(String PortName, int Baud) 
        {
            // Create a new SerialPort object with default settings.
            // Open it, too.
            mbedPort = new SerialPort();

            // Set the read/write timeouts
            mbedPort.ReadTimeout = 500;
            mbedPort.WriteTimeout = 500;

            mbedPort.PortName = PortName;
            mbedPort.BaudRate = Baud;

            // always the same
            mbedPort.DataBits = 8;
            mbedPort.Parity = Parity.None;
            mbedPort.StopBits = StopBits.One;

            mbedPort.Open();

            mbedPort.ErrorReceived += new SerialErrorReceivedEventHandler(mbedPort_ErrorReceived);
	    }

	    /**
	     * {@inheritDoc}
	     */
	    public override String RPC(String Name, String Method, String[] Args)
        {
		    //write to serial port and receive result
		    String Arguments = String.Empty;
		    String response = String.Empty;

		    if(Args != null)
            {
			    for(int i = 0; i < Args.Length; i++)
                {
				    Arguments = Arguments + " " + Args[i];
			    }
		    }
		    String RPCString = "/" + Name + "/" + Method + Arguments + "\n";

			mbedPort.Write(RPCString);

		    bool valide = true;

            while (valide)
            {
                response = mbedPort.ReadLine();
                valide = false;
            }
 		    return(response);
	    }

        private String ErrMsg(String message)
        {
            //Error reading a line from the port
            Debug.Print("Error reading line: " + message);
            return "error";
        }

	    /**
	     * {@inheritDoc}
	     */
	    public override void delete()
        {
		    //Close the serial port
            if (mbedPort != null) mbedPort.Close();
	    }

        private void mbedPort_ErrorReceived(object sender, EventArgs e)
        {
            Debug.Print("Serial Port error");
        }
    }
}
